package com.yunzhi.nanyang.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyV3Result;
import com.github.binarywang.wxpay.bean.notify.WxPayRefundNotifyV3Result;
import com.github.binarywang.wxpay.bean.request.WxPayRefundV3Request;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderV3Request;
import com.github.binarywang.wxpay.bean.result.WxPayUnifiedOrderV3Result;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.constant.WxPayErrorCode;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.yunzhi.nanyang.common.Constants;
import com.yunzhi.nanyang.common.DateUtils;
import com.yunzhi.nanyang.common.StringUtils;
import com.yunzhi.nanyang.common.WxPayUtils;
import com.yunzhi.nanyang.entity.TaOrder;
import com.yunzhi.nanyang.entity.TaPay;
import com.yunzhi.nanyang.mapper.TaOrderMapper;
import com.yunzhi.nanyang.mapper.TaPayMapper;
import com.yunzhi.nanyang.mapper.TaPersonMapper;
import com.yunzhi.nanyang.service.ITaPayService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

@Slf4j
@Service
public class TaPayServiceImpl extends BaseServiceImpl<TaPayMapper, TaPay> implements ITaPayService {


    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Autowired
    WxPayUtils wxPayUtils;

    @Override
    public WxPayUnifiedOrderV3Result.JsapiResult createPay(String client, String clientId, TaOrder taOrder) throws Exception {
        TaPay taPay = getByOrderNo(taOrder.getOrderNo());
        if (null == taPay) {
            taPay = new TaPay();
            taPay.setOrderId(taOrder.getOrderId());
            taPay.setOrderNo(taOrder.getOrderNo());
            taPay.setPersonId(taOrder.getPersonId());
            taPay.setCharges(taOrder.getCharges());

            if (taOrder.getTypeName().length() > 127) {
                taPay.setDescription(taOrder.getTypeName().substring(0, 127));
            } else {
                taPay.setDescription(taOrder.getTypeName());
            }

            taPay.setPayType(taOrder.getPayType());
            taPay.setStatus(Constants.PAY_READY);
        }

        try {
            WxPayUnifiedOrderV3Result.JsapiResult jsapiResult = prePay(taPay);
            if (StringUtils.isEmpty(taPay.getPayId())) {
                taPay.setPayId(StringUtils.uuid());
                if (!save(taPay)) {
                    throw new Exception("系统出错请重试");
                }
            }
            return jsapiResult;
        } catch (WxPayException e) {
            e.printStackTrace();

            boolean isPaid = WxPayErrorCode.UnifiedOrder.ORDERPAID.equals(e.getErrCode());
            if (isPaid) {
                // 如果订单已支付
                taPay.setStatus(Constants.PAY_PAID);
                taPay.setPayDate(LocalDateTime.now());
                if (StringUtils.isEmpty(taPay.getPayId())) {
                    taPay.setPayId(StringUtils.uuid());
                    if (!save(taPay)) {
                        throw new Exception("系统出错请重试");
                    }
                } else {
                    updateById(taPay);
                }
                taOrderMapper.updatePayStatus(taPay.getOrderId(), Constants.PAY_PAID);
            }

            throw e;
        }
    }

    @Override
    public WxPayUnifiedOrderV3Result.JsapiResult prePay(TaPay taPay) throws WxPayException {
        // 生成订单
        if ("wx".equals(taPay.getPayType())) {
            WxPayUnifiedOrderV3Request.Amount amount = new  WxPayUnifiedOrderV3Request.Amount();
            amount.setTotal(taPay.getCharges());

            WxPayUnifiedOrderV3Request.Payer payer = new WxPayUnifiedOrderV3Request.Payer();
            payer.setOpenid(taPersonMapper.selectById(taPay.getPersonId()).getOpenid());

            WxPayUnifiedOrderV3Request req = new WxPayUnifiedOrderV3Request();
            req.setDescription(taPay.getDescription());
            req.setOutTradeNo(taPay.getOrderNo());
            req.setAmount(amount);
            req.setPayer(payer);

            return wxPayUtils.createOrder(req);
        }

        return null;
    }

    @Override
    public TaPay getByOrderNo(String orderNo) {
        QueryWrapper<TaPay> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("order_no", orderNo);
        queryWrapper.gt("status", Constants.STATUS_DELETE);

        return getOne(queryWrapper);
    }

    @Override
    public void notifyOrder(WxPayOrderNotifyV3Result.DecryptNotifyResult result) {
        TaPay orderPay = getByOrderNo(result.getOutTradeNo());
        if (orderPay.getStatus() != Constants.PAY_READY) {
            return;
        }

        boolean isSuccess = WxPayConstants.ResultCode.SUCCESS.equals(result.getTradeState());

        orderPay.setPayDate(DateUtils.from(result.getSuccessTime(), DateTimeFormatter.ISO_DATE_TIME));
        orderPay.setStatus(isSuccess ? Constants.PAY_PAID : Constants.PAY_FAIL);
        orderPay.setMessage(isSuccess ? null : result.getTradeStateDesc());
        baseMapper.updateById(orderPay);

        // 更新订单状态
        if (isSuccess) {
            taOrderMapper.updatePayStatus(orderPay.getOrderId(), Constants.PAY_PAID);
        }

        // 插入账户流水
        // 暂无
    }

    @Override
    public void refund(String client, String clientId, TaOrder taOrder) throws Exception {
        TaPay origin = getByOrderNo(taOrder.getOrderNo());
        if (null == origin) {
            throw new Exception("未找到支付信息");
        }

        if (origin.getStatus() != Constants.PAY_PAID) {
            throw new Exception("订单支付校验失败");
        }

        String refundNo = "R-"+origin.getOrderNo();
        TaPay taPay = getByOrderNo(refundNo);
        if (null == taPay) {
            taPay = new TaPay();
            taPay.setOrderId(taOrder.getOrderId());
            taPay.setOrderNo(refundNo);
            taPay.setPersonId(taOrder.getPersonId());
            taPay.setCharges(taOrder.getCharges());
            taPay.setIsRefund(true);
            taPay.setDescription("退款");
            taPay.setPayType(taOrder.getPayType());
            taPay.setStatus(Constants.PAY_READY);
        }

        WxPayRefundV3Request request = new WxPayRefundV3Request();
        request.setOutTradeNo(origin.getOrderNo()); // 原单号
        request.setOutRefundNo(taPay.getOrderNo()); // 退款单号
        WxPayRefundV3Request.Amount amount = new  WxPayRefundV3Request.Amount();
        amount.setTotal(taPay.getCharges());
        amount.setRefund(taPay.getCharges());
        amount.setCurrency("CNY");
        request.setAmount(amount);

        wxPayUtils.refund(request);
        if (StringUtils.isEmpty(taPay.getPayId())) {
            // 退款申请中 ...
            taOrder.setPayStatus(Constants.PAY_REFUNDING);
            taOrderMapper.updateById(taOrder);

            taPay.setPayId(StringUtils.uuid());
            if (!save(taPay)) {
                throw new Exception("内部错误, 请重新操作");
            }
        }
    }

    @Override
    public void refundNotifyOrder(WxPayRefundNotifyV3Result.DecryptNotifyResult result) {
        TaPay taPay = getByOrderNo(result.getOutRefundNo());
        if (null == taPay) {
            log.error("退款通知处理失败: 单号 " + result.getOutRefundNo());
            log.error(result.toString());
            return;
        }

        if (taPay.getStatus() != Constants.PAY_READY) {
            return;
        }

        boolean isSuccess = WxPayConstants.RefundStatus.SUCCESS.equals(result.getRefundStatus());

        taPay.setPayDate(DateUtils.from(result.getSuccessTime(), DateTimeFormatter.ISO_DATE_TIME));
        taPay.setStatus(isSuccess ? Constants.PAY_PAID : Constants.PAY_FAIL);
        taPay.setMessage(isSuccess ? null : result.getRefundStatus());
        baseMapper.updateById(taPay);

        // 更新订单状态
        if (isSuccess) {
            TaOrder taOrder = taOrderMapper.selectById(taPay.getOrderId());
            taOrder.setPayStatus(Constants.PAY_REFUND);
            taOrder.setIsRefund(1);
            taOrder.setStatus(Constants.ORDER_CANCEL);
            taOrderMapper.updateById(taOrder);
        }

        // 插入账户流水
        // 暂无

    }


}
